library(ggplot2)
library(reshape2)
library(lattice)
library(dplyr)

file <- c("FMIB", "CAAB", "FMR1", "FMR2", "FOIB", "FOAB", "FOR1", "FOR2")
site <- c("RBIB", "RBAB1", "RBR1", "RBR2", "BBIB", "BBAB", "BBR1", "BBR2")
path <- "../streamflow/"
g_df <- NULL

for (i in 1:length(file)) {
  
  print(file[i])
  
  # Read in precipitation (P) and discharge (Q)
  pqdat <- read.table(paste(path, file[i], ".csv", sep=""), header = TRUE, sep = ",", as.is = TRUE)
  
  # Drop P column
  pqdat$P <- NULL
  
  # Calculate mean q
  meanQ <- mean(pqdat$Q, na.rm = TRUE)
  
  # Divide by mean for catchment
  pqdat$Q <- ((pqdat$Q / meanQ) + 1)
  
  # Convert to log
  pqdat$LogQ <- log10(pqdat$Q)
  
  # Add site field
  pqdat$Site <- site[i]
  if (i <= 4) {
    pqdat$Location <- "RB"
  }
  else {
    pqdat$Location <- "BB"
  }
  
  # Take out date
  qdat <- subset(pqdat, select=c("Location", "Site", "LogQ"))
  
  # Bind rows
  g_df <- bind_rows(qdat, g_df)
}

# Define breaks
pbreaks <- c(0.0001, 0.01, 0.1, 0.3, 0.5, 0.8, 0.95, 0.99, 0.9999)  

# Set site factor
g_df$Site <- factor(g_df$Site, levels=c("RBIB", "RBAB1", "RBR1", "RBR2", "BBIB", "BBAB", "BBR1", "BBR2"))

# Set location factor
g_df$Location <- factor(g_df$Location, levels=c("RB", "BB"))


# Colour blind friendly palette
cbPalette <- c("#999999", "#E69F00", "#56B4E9", "#009E73", "#999999", "#E69F00", "#56B4E9", "#009E73")

# Plot log Q against log probability scale
p <- ggplot(g_df, aes(sample=LogQ, shape = Site , colour = Site)) + 
  scale_fill_manual(name="Site", values=cbPalette) +
  scale_shape_manual(name="Site", values=c(0, 1, 2, 3, 4, 5, 6, 7, 8)) +
  scale_colour_manual(name="Site", values=cbPalette) +
  stat_qq(geom="line", na.rm = TRUE, size=1) +
  theme_light() +
  scale_x_continuous(labels = prettyNum(pbreaks*100), breaks=qnorm(pbreaks)) +
  labs(x = "Percent time flow not exceeded", y = expression('log(1 + Q/mean(Q))')) +
  facet_wrap(facets = ~Location, labeller = labeller(Location = c("RB" = "Raised bog", "BB" = "Blanket bog"))) +
  theme_bw(base_size = 14) + theme(legend.position = "bottom", axis.text.y = element_text(hjust=0)) +
  guides(colour = guide_legend(override.aes = list(size=4)))

p

ggsave(filename="fdc4.pdf", 
       plot = p, 
       device = cairo_pdf, 
       width = 210, 
       height = 100, 
       units = "mm")







