library(zoo)
library(ggplot2)
require(reshape2)
library(dplyr)
library(KFAS)
library(tseries)
library(timeSeries)
library(TSstudio)

path <- "../dipwells/"
file <- c("FOIB", "FMAB", "CAAB", "FMR1", "FMR2_pooled", "FOIB", "FOAB", "FOR1", "FOR2")
site <- c("RBIB", "RBAB2", "RBAB1", "RBR1", "RBR2", "BBIB", "BBAB", "BBR1", "BBR2")
dfMonthly <- data.frame()
dfYearly <- data.frame()
dfSurface <- data.frame()

# Set time zone
Sys.setenv(TZ = "GMT")

# Zoo object to data frame helper function
zoo.to.data.frame <- function(x, index.name="Date") {
  stopifnot(is.zoo(x))
  xn <- if(is.null(dim(x))) deparse(substitute(x)) else colnames(x)
  setNames(data.frame(index(x), x, row.names=NULL), c(index.name,xn))
}

smooth <- function(ts, factor = 0.01) {
  
  # Create ts object for KFS filter
  t <- zoo_to_ts(ts)
  
  # Create model
  ssModel <- SSModel(t ~SSMtrend(1, Q = factor), H=(factor), distribution = "gaussian")
  
  # SSM fit
  ssModel <- fitSSM(ssModel, inits = c(0, 0), method = "BFGS")$model
  
  # Smooth
  kfs <- KFS(ssModel, smoothing="state", nsim=length(t), maxiter = 20000)
  
  # Confidence and prediction intervals for the expected value and the observations.
  # Note that predict uses original model object, not the output from KFS.
  confSSM <- predict(ssModel, interval = "confidence", level = 0.90)
  predSSM <- predict(ssModel, interval = "prediction", level = 0.90)
  
  # Mean smooth
  tNA <- t
  tNA <- SSModel(tNA ~ SSMtrend(1, Q = list(ssModel$Q)))
  kfsNA <- KFS(tNA, "mean", "mean")
  
  # Fitted
  kfsfit <- fitted(kfsNA)
  
  # Create fitted time series
  kfs_ts <- as.zoo(kfsfit)
  
  # Create kfs data frame
  kfs_df <- zoo.to.data.frame(kfs_ts)
  
  # Return smoothed data frame
  return(kfs_df)
}

aggregate_wtd <- function(i, tsWTD, by="months") {
  
  if (by == "year") {
    # Aggregate by month - mean
    meanWTD <- mean(tsWTD$WTD)
    
    # Aggregate by month - standard deviation
    meanSD <- sd(tsWTD$WTD)
    
    # Aggregate by month - se
    meanSE <- sd(tsWTD$WTD) / sqrt(length(tsWTD$WTD))
    
    # Aggregate by month
    minWTD <- min(tsWTD$WTD)
    
    # Aggregate by month
    maxWTD <- max(tsWTD$WTD)
    
    # Aggregate % time at surface
    atSurface <- (sum(tsWTD$WTD >= 0) / sum (tsWTD$WTD < 500))*100
    
    # Create data frame of dates
    df <- data.frame(Site = site[i], mean = meanWTD, sd = meanSD, se = meanSE, min = minWTD, max = maxWTD, atSurface = atSurface)
  }
  else {
    # Aggregate by month - mean
    meanWTD <- aggregate(list(mean = tsWTD$WTD), list(month = cut(as.Date(time(tsWTD)), by)), mean)
    
    # Aggregate by month - standard deviation
    meanSD <- aggregate(list(sd = tsWTD$WTD), list(month = cut(as.Date(time(tsWTD)), by)), function(x) sd = sd(x))
    
    # Aggregate by month - se
    meanSE <- aggregate(list(se = tsWTD$WTD), list(month = cut(as.Date(time(tsWTD)), by)), function(x) se = sd(x) / sqrt(length(x)))
    
    # Aggregate by month
    minWTD <- aggregate(list(min = tsWTD$WTD), list(month = cut(as.Date(time(tsWTD)), by)), min)
    
    # Aggregate by month
    maxWTD <- aggregate(list(max = tsWTD$WTD), list(month = cut(as.Date(time(tsWTD)), by)), max)
    
    # Aggregate % time at surface
    atSurface <- aggregate(list(count = tsWTD$WTD), list(month = cut(as.Date(time(tsWTD)), by)), function(x) count = (sum(x >= 0) / sum (x < 500))*100)
    
    # Create data frame of dates
    df <- data.frame(Date = as.Date(meanWTD[,1]), Site = site[i], mean = meanWTD[,2], sd = meanSD[,2], se = meanSE[,2], min = minWTD[,2], max = maxWTD[,2], atSurface = atSurface[,2])
  }
  
  # Return data frame
  return (df)
}

for (i in 1:length(file)) {
  
  # Read in precipitation (P) and discharge (WTD)
  wtdat <- read.table(paste(path, file[i], ".csv", sep=""), header = TRUE, sep = ",", as.is = TRUE)
  
  # Format date_times
  wtdat$Date <- as.POSIXct(wtdat$Date, "%d/%m/%Y %H:%M", tz="GMT")
  
  # Create time series for P and WTD
  tsWTD <- zoo(x = wtdat[, 2:3], order.by = wtdat$Date)
  
  # Copy in smoothed WTD
  tsWTD$WTD <- smooth(tsWTD$WTD, factor = 0.001)[, 2]
  
  # % time at surface
  dft <- zoo.to.data.frame(tsWTD)
  dfs <- subset(dft, WTD <= 0)
  atSurface <- (length(dfs$WTD) / length(dft$WTD) * 100)
  dfRow <- data.frame(Site = site[i], AtSurface = atSurface)
  dfSurface <- bind_rows(dfSurface, dfRow)
  
  # Set hydrological year
  #tsWTD <- window(tsWTD, start=as.POSIXct("2018-10-01", tz="GMT"), end=as.POSIXct("2019-09-30", tz="GMT"))
  
  # Get monthly summary
  df <- aggregate_wtd(i, tsWTD)
  
  # Bind rows
  dfMonthly <- bind_rows(dfMonthly, df)
  
  # Get annual summary
  df <- aggregate_wtd(i, tsWTD, by="year")
  
  # Bind rows
  dfYearly <- bind_rows(dfYearly, df)
}

# Colour blind friendly palette
cbPalette <- c("#999999", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7")

# Monthly plot means
p <- ggplot(data = dfMonthly,
  mapping = aes(x = Date, y = mean, ymin = mean - sd, ymax = mean + sd, col="Mean (+/-) SD")) +
  scale_colour_manual(values=cbPalette, name="WTD") +
  scale_y_reverse() +
  geom_point(shape=0) +
  geom_smooth(stat = "identity") +
  geom_line() +
  geom_point(shape=1, aes(y = min, col="Min")) +
  geom_line(aes(y = min, col="Min")) +
  geom_point(shape=2, aes(y = max, col="Max")) +
  geom_line(aes(y = max, col="Max")) +
  scale_x_date(date_labels = "%b%y", date_breaks = "2 months") +
  facet_wrap(facets = ~Site) +
  labs(y = "WTD (cm)") +
  theme_bw(base_size = 20) + 
  theme(legend.position = "bottom", axis.text.x = element_text(size=14, angle = 45, hjust = 1))

p

ggsave(filename="monthly-wtd-metrics-smooth-RB-BB.pdf", 
       plot = p, 
       device = cairo_pdf, 
       width = 297, 
       height = 210, 
       units = "mm")

# Write monthly CSV
write.csv(dfMonthly, "wtd-monthly-summary-smooth.csv")

# Write annual CSV
write.csv(dfYearly, "wtd-annual-summary-smooth.csv")

# Write annual CSV
write.csv(dfSurface, "atSurface.csv")






