library(foreign)
library(ggplot2)
library(dplyr)
library(ggpubr)
library(reshape2)
library(data.table)
library(ggpubr)

# Set path
path <- "../dipwells/"

# Read in storm metrics file
dfAll <- read.table(paste(path, "wt_metrics", ".csv", sep=""), header = TRUE, sep = ",", as.is = TRUE)

# Plot labels
plot_labels <- c(
  'PeakWTD' = 'Peak~WTD~(cm)',
  'PeakLag' = 'Peak~lag~(h)',
  'Dur' = 'WTD~rise~dur~(h)',
  'Recess6' = 'Recess6~(cm~h^-1)',
  'Recess12' = 'Recess12~(cm~h^-1)',
  'WTRise' = 'WT~rise~(cm)',
  'FM' = 'Flanders~Moss',
  'FO' = 'Forsinain'
)

# Select variables
df.no <- select(dfAll, "Site", "Location", "WTRise", "PeakWTD", "Dur", "PeakLag", "Recess6", "Recess12")

# Set site factor
df.no$Site <- factor(df.no$Site, levels=c("FMIB", "CAAB", "FMAB", "FMR1", "FMR2", "", "FOIB", "FOAB", "FOR1", "FOR2"))

df.s <- df.no

# Format extreme outliers for peakWTD
dd <- df.s %>% dplyr::filter(PeakWTD >= 35)
dd <- as.data.table(dd)[, list(toString(format(round(max(PeakWTD), 1), nsmall = 1), width=8), colnames(df.s[4]), 35), by = list(Location, Site)]

# Recess6
dd2 <- df.s %>% dplyr::filter(Recess6 >= 0.8)
dd2 <- as.data.table(dd2)[, list(toString(format(round(max(Recess6), 1), nsmall = 1), width=8), colnames(df.s[7]), 0.8), by = list(Location, Site)]

# Recess12
dd3 <- df.s %>% dplyr::filter(Recess12 >= 0.8)
dd3 <- as.data.table(dd3)[, list(toString(format(round(max(Recess12), 1), nsmall = 1), width=8), colnames(df.s[8]), 0.8), by = list(Location, Site)]

# Column names
colnames(dd) <- c("Location", "Site", "lbl", "variable", "Max")
colnames(dd2) <- c("Location", "Site", "lbl", "variable", "Max")
colnames(dd3) <- c("Location", "Site", "lbl", "variable", "Max")

df.N <- aggregate(WTRise~Site, data=df.no, FUN=function(x) c(length = length(x)))

colnames(df.N) <- c("Site", "N")

# Filter out extreme outliers
df.no <- df.no[!is.na(df.no$PeakWTD) & df.no$PeakWTD < 35,]
df.no <- df.no[!is.na(df.no$Recess6) & df.no$Recess6 < 0.8,]
df.no <- df.no[!is.na(df.no$Recess12) & df.no$Recess12 < 0.8,]

# Colour blind friendly palette
cbPalette <- c("#999999", "#E69F00", "#808000", "#56B4E9", "#009E73", "#999999", "#E69F00", "#56B4E9", "#009E73")

# Create theme
mytheme <- theme_bw(base_size = 20) + theme(
  legend.position = "none", 
  axis.ticks.x = element_blank(), 
  axis.title.x = element_blank(),
  plot.margin = unit(c(0.2,0.3,0.2,0.3), "cm")
)

p1 <- ggplot(data = df.no, aes(x=Site, y=WTRise)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab("WT rise (cm)") +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBAB2", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  mytheme + theme(axis.text.x = element_blank())

p2 <- ggplot(data = df.no, aes(x=Site, y=PeakWTD)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  scale_y_reverse() +
  ylab("Peak WTD (cm)") +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBAB2", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  geom_text(data=dd, aes(x=factor(Site), y=Max, label=lbl), size=3.5) +
  geom_segment(data=dd, aes(y=Max*0.85,yend=Max*0.95, xend=factor(Site)), arrow = arrow(length = unit(0.1,"cm"))) +
  mytheme + theme(axis.text.x = element_blank())

p3 <- ggplot(data = df.no, aes(x=Site, y=Dur)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab("Rise dur (h)") +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBAB2", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  mytheme + theme(axis.text.x = element_blank())

p4 <- ggplot(data = df.no, aes(x=Site, y=PeakLag)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab("Peak lag (h)") +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBAB2", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  mytheme + theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1))

p5 <- ggplot(data = df.no, aes(x=Site, y=Recess6)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab(expression(R6~(cm ~h^{-1}))) +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBAB2", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  geom_text(data=dd2, aes(x=factor(Site), y=Max, label=lbl), size=3.5) +
  geom_segment(data=dd2, aes(y=Max*0.85,yend=Max*0.95, xend=factor(Site)), arrow = arrow(length = unit(0.1,"cm"))) +
  mytheme + theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1))

p6 <- ggplot(data = df.no, aes(x=Site, y=Recess12)) + geom_boxplot(aes(fill=Site), notch=TRUE) +
  scale_fill_manual(name="Site", values=cbPalette) +
  ylab(expression(R12~(cm ~h^{-1}))) +
  stat_summary(fun.y="mean", color="black", shape=23) +
  scale_x_discrete(labels = c("RBIB", "RBAB1", "RBAB2", "RBR1", "RBR2", "", "BBIB", "BBAB", "BBR1", "BBR2"), drop=FALSE) +
  geom_vline(xintercept = "") +
  geom_text(data=dd3, aes(x=factor(Site), y=Max, label=lbl), size=3.5) +
  geom_segment(data=dd3, aes(y=Max*0.85,yend=Max*0.95, xend=factor(Site)), arrow = arrow(length = unit(0.1,"cm"))) +
  mytheme + theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1))

# Cow plot arrange
plots <- cowplot::plot_grid(p1, p2, p3, p4, p5, p6, nrow=2, ncol=3, labels = c('a', 'b', 'c', 'd', 'e', 'f'), label_size = 18, align = "v", rel_heights = c(0.4, 0.5))

df.N <- t(df.N)

# Format table
stable <- ggtexttable(df.N, theme = ttheme("light", base_size = 16))

fig <- cowplot::plot_grid(plots, stable, nrow=2, ncol=1, rel_heights = c(0.90, 0.10))

ggsave(filename="wt_metrics_bp_no_facets_RB_BB_rev.pdf", 
       plot = fig, 
       device = cairo_pdf, 
       width = 297, 
       height = 200, 
       units = "mm")
