library(Hmisc)
library(zoo)
library(ggplot2)
library(cowplot)
library(KFAS)
library(tseries)
library(timeSeries)
library(TSstudio)
library(dplyr)
library(lubridate)

path <- "../dipwells/"
site <- c("FOIB", "FOAB", "FOR1", "FOR2")

dfAll <- data.frame()

# Set time zone
Sys.setenv(TZ = "GMT")

# Zoo object to data frame helper function
zoo.to.data.frame <- function(x, index.name="Date") {
  stopifnot(is.zoo(x))
  xn <- if(is.null(dim(x))) deparse(substitute(x)) else colnames(x)
  setNames(data.frame(index(x), x, row.names=NULL), c(index.name,xn))
}

smooth <- function(ts, factor = 0.01) {
  
  # Create ts object for KFS filter
  t <- zoo_to_ts(ts)
  
  # Create model
  ssModel <- SSModel(t ~SSMtrend(1, Q = factor), H=(factor), distribution = "gaussian")
  
  # SSM fit
  ssModel <- fitSSM(ssModel, inits = c(0, 0), method = "BFGS")$model
  
  # Smooth
  kfs <- KFS(ssModel, smoothing="state", nsim=length(t))
  
  # Confidence and prediction intervals for the expected value and the observations.
  # Note that predict uses original model object, not the output from KFS.
  confSSM <- predict(ssModel, interval = "confidence", level = 0.90)
  predSSM <- predict(ssModel, interval = "prediction", level = 0.90)
  
  # Mean smooth
  tNA <- t
  tNA <- SSModel(tNA ~ SSMtrend(1, Q = list(ssModel$Q)))
  kfsNA <- KFS(tNA, "mean", "mean")
  
  # Fitted
  kfsfit <- fitted(kfsNA)
  
  # Create fitted time series
  kfs_ts <- as.zoo(kfsfit)
  
  # Create kfs data frame
  kfs_df <- zoo.to.data.frame(kfs_ts)
  
  # Return smoothed data frame
  return(kfs_df)
}

for (i in 1:length(site)) {
  
  # Read in precipitation (P) and discharge (WTD)
  wtdat <- read.table(paste(path, site[i], ".csv", sep=""), header = TRUE, sep = ",", as.is = TRUE)
  
  # Format date_times
  wtdat$Date <- as.POSIXct(wtdat$Date, format="%d/%m/%Y %H:%M", tz="GMT")
  
  # Create time series of 15 min time steps
  tsWTD <- zoo(x = wtdat[, 2:3], order.by = wtdat$Date)
  
  # Copy in smoothed WTD
  #wtdat$WTD <- smooth(tsWTD$WTD, factor = 0.001)[, 2]
  
  # Start time
  start <- as.POSIXct("02/03/2018", "%d/%m/%Y", tz="GMT")
  end <- start + as.difftime(641, units="days")
  
  # Create empty timeseries
  seq <- seq(from=start, by=60*15, to=end)
  
  # Create NA data frame
  dfNA <- data.frame(Date = seq, Val=1)
  
  # Create NA time series
  tsNA <- zoo(x = dfNA[, 2:2], order.by = dfNA$Date)
  
  # Set date range
  tsWTD <- window(tsWTD, start=start, end=end)
  
  # Merge into one time-series
  tsMerge <- zoo::merge.zoo(tsNA, tsWTD)
  
  # Convert to data frame
  df <- zoo.to.data.frame(tsMerge)
  
  # Format colnames
  colnames(df) <- c("Date", "tsNA", "P", paste(site[i], "_wtd", sep=""))
  
  # Always drop tsNA - not needed
  df$tsNA <- NULL
  
  # If not at start only include first Date column
  if (i != 1) {
    df$Date <- NULL
    df$P <- NULL
  }
  
  if (i == 1) {
    # Init FM frame
    dfAll <- df
  }
  else {
    # Bind FM columns
    dfAll <- cbind(dfAll, df)   
  }
}

# Convert to POSIX date format
dfAll$Date <- as.POSIXct(dfAll$Date, tz="GMT")

# Colour blind friendly palette
cbPalette <- c("#999999", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7")

dfWTD <- dfAll

colnames(dfWTD) <- c("Date", "P", "FOIB", "FOAB", "FOR1", "FOR2")

# Correlate
c <- rcorr(as.matrix(dfWTD[3:6]), type = "spearman")

# WTD CSV
write.csv(c$r, "wtd_fo_corr.csv")









